/*
 * Copyright (C) 2011, Mukesh Rathor, Oracle Corp.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License v2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 021110-1307, USA.
 */
#ifndef __XEN_PUBLIC_XMSTAT_H__
#define __XEN_PUBLIC_XMSTAT_H__

#ifdef __XEN__
#include <asm/percpu.h>
#else
#define likely(x)     __builtin_expect((x),1)
#define unlikely(x)   __builtin_expect((x),0)
#endif

#define XMST_MAX_CPUS 1024                  /* max cpus xmstat can support */
#define XMST_PCPU_MAX_PGS 32                /* max num of pgs for per cpu data*/
#define XMST_INVALID_DOMID (0xFFFFFFFFU)    /* reflects idle vcpu */


enum {
    XMSTAT_INIT = 1,      /* initialize xmstat */
    XMSTAT_PG_INFO,       /* send mfns info to user level */
    XMSTAT_DEINIT,        /* stop, end, and cleanup */
};

struct xmst_init {
    int      xmst_interval_us;        /* IN: cpu sample interval in micro secs*/
    int      xmst_debug_on;           /* IN: turn on debug mode */
    int      xmst_num_data_pgs;       /* IN: num of per cpu data pages */
    int      xmst_num_cpus;           /* OUT: number of cpus being traced */
};

struct xmst_pginfo {
    int      xmst_cpunum;                  /* IN: cpu num. -1 == meta info */
    int      xmst_num_mfns;                /* OUT: num pages in xmst_mfna */
    uint64_t xmst_mfna[XMST_PCPU_MAX_PGS]; /* OUT: array of mfns */
};

struct xen_perfop_xmstat {
    int xmst_cmd;                     /* XMSTAT_INIT etc.. */
    union {
        struct xmst_init xmst_init;
        struct xmst_pginfo xmst_pginfo;
    } u;
};


/* per cpu xmstat meta data */
struct xmst_pcpu_meta {
    uint32_t xmst_cpuid;          /* physical cpu id */
    uint32_t xmst_index;          /* index of next xmst_pcpu_data entry */
    uint32_t xmst_pfaults;        /* number of page faults */
    uint32_t xmst_interrupts;     /* number of interrupts */
    uint32_t xmst_hcalls;         /* number of hypercalls */
    uint32_t xmst_evtchns;        /* number of event channel ops */
};

#define MAX_IDX_VAL (~(uint32_t)0)      /* xmst_index rollover val */

/* each cpu page has an array of these where samples are taken . size must be
 * power of 2 as indexs are & with XMST_MAX_IDX */
struct xmst_pcpu_data {
    uint32_t xmst_domid;       /* domain id */
    uint32_t xmst_vcpu;        /* vcpu id */
};

#define XMST_PGSZ 4096         /* not avail in public header */

#ifdef __XEN__
DECLARE_PER_CPU(struct xmst_pcpu_meta *, xmst_pcpu_mp);
extern volatile unsigned int xperf_active;
#endif

#endif /*  __XEN_PUBLIC_XMSTAT_H__ */
